#!/bin/sh

if [ -z "${__included_mount_funcs-}" ]; then
__included_mount_funcs=1

. boot-func.sh

ROOT_MPOINT=/
SYSDIRS="/proc /sys /dev /dev/pts"
SDDIRS="/mnt/storage"
TMPCPDIRS="/etc/dhcpc"

is_mounted() {
  local arg="$1"
  if is_bootname "$arg"; then
      local dev="$(get_root_device "$arg")"
      mount -l | grep -q "^$dev[[:space:]]"
  else
      mount -l | grep -qw "$arg"
  fi
}

get_root_mpoint()
{
    local name="$1"

    case "$name" in
        NAND)
            echo "/mnt/nand-root"
            ;;
        SD*)
            echo "/mnt/sd-root${name#SD}"
            ;;
        *)
            echo "Unknown boot device: $1" >&2
            return 1
            ;;
    esac
}

get_mpoint_name() {
    local mpoint="$1"

    case "$mpoint" in
        /mnt/nand-root)
            echo 'NAND'
            ;;
        /mnt/sd-root*)
            echo "SD${mpoint#/mnt/sd-root}"
            ;;
        *)
            echo 'UNKNOWN'
            return 1
            ;;
    esac
}

is_nand_mpoint() {
    [ "$(get_mpoint_name "${1:-}")" = 'NAND' ]
}

is_sd_mpoint() {
    case "$(get_mpoint_name "${1:-}")" in
        SD*)
            return 0
            ;;
    esac
    
    return 1
}

attach_nand() {
    local dev="${1:-$(get_root_device NAND)}"
    if [ ! -e /dev/ubi0 ]; then
        ubiattach /dev/ubi_ctrl -m $ROOT_NAND_NUM
    fi
}

detach_nand() {
    local dev="${1:-$(get_root_device NAND)}"
    if [ -e /dev/ubi0 ]; then
        ubidetach /dev/ubi_ctrl -m $ROOT_NAND_NUM
    fi
}

mountfs()
{
  local dev="$1"; shift
  if is_bootname "$dev"; then
      local mpoint="$(get_root_mpoint "$dev")"
      dev="$(get_root_device "$dev")"
  else
      local mpoint="$1"; shift
  fi
  
  if ! is_mounted "$dev";  then
    mkdir -p "$mpoint"
    case "$(get_device_name "$dev")" in
        NAND)
            attach_nand "$dev"
            mount -t ubifs ubi0:rootfs "$@" "$mpoint"
            ;;
        SD*)
            mount "$@" "$dev" "$mpoint"
            ;;
        *)
            echo "Unsupported device: $dev" >&2
            return 1
            ;;
    esac
  fi
}

bind_dir() {
    local dir="$1"
    local mpoint="$2"

    if [ -d "$dir" ]; then
        if ! is_mounted "${mpoint%/}/${dir#/}"; then
            mkdir -p "${mpoint%/}/${dir#/}"
            mount -o bind "$dir" "${mpoint%/}/${dir#/}"
        fi
    fi
}

bind_dirs() {
    local mpoint="$1"; shift

    for dir in "$@"; do
        bind_dir "$dir" "$mpoint"
    done
}

bind_tmp() {
    local mpoint="$1"
    mpoint="${mpoint%/}"
    tmpdir="/tmp/.rootdir/${mpoint#/}"
    
    mkdir -p "$tmpdir"
    mkdir -p "$mpoint/tmp"
    mount -o bind "$tmpdir" "$mpoint/tmp"

    for dir in $TMPCPDIRS; do
        dir="${dir#/}"
        mkdir -p "$mpoint/tmp/${dir%/}"
        cp -af "/tmp/${dir%/}/." "$mpoint/tmp/${dir%/}/"
    done
}

ubind_dir() {
    local dir="$1"

    if [ -d "$dir" ] && is_mounted "$dir"; then
        umount "$dir"
    fi
}

ubind_dirs() {
    local mpoint="$1"; shift

    for dir in "$@"; do
        ubind_dir "${mpoint%/}/${dir#/}"
    done
}

ubind_tmp() {
    local mpoint="$1"
    mpoint="${mpoint%/}"
    tmpdir="/tmp/.rootdir/${mpoint#/}"
 
    umount "$mpoint/tmp"
    rm -rf "$tmpdir"
}

mountrootfs() {
    mountfs "$@"

    local dev="$1"; shift
    if is_bootname "$dev"; then
        local mpoint="$(get_root_mpoint "$dev")"
        dev="$(get_root_device "$dev")"
    else
        local mpoint="$1"; shift
    fi

    bind_dirs "$mpoint" $SYSDIRS $SDDIRS
    bind_tmp "$mpoint"
}

umountfs()
{
  local dev="$1"; shift
  if is_bootname "$dev"; then
      local mpoint="$(get_root_mpoint "$dev")"
      dev="$(get_root_device "$dev")"
  else
      local mpoint="${1:-}"
      [ $# -eq 0 ] || shift
  fi
  
  if [ -d "$dev" ]; then
      mpoint="$dev"
      if is_mounted "$mpoint";  then
          umount "$@" "$mpoint"
          case "$(get_mpoint_name "$mpoint")" in
              NAND)
                  detach_nand
                  ;;
          esac
      fi
  else
      if is_mounted "$dev";  then
          umount "$@" "$mpoint"
          case "$(get_device_name "$dev")" in
              NAND)
                  detach_nand "$dev"
                  ;;
              SD*)
                  ;;
              *)
                  echo "Unsupported device: $dev" >&2
                  return 1
                  ;;
          esac
      fi
  fi
}

umountrootfs() {
    local dev="$1"
    if is_bootname "$dev"; then
        local mpoint="$(get_root_mpoint "$dev")"
        dev="$(get_root_device "$dev")"
    else
        local mpoint="${2:-}"
    fi

    ubind_tmp "$mpoint"
    ubind_dirs "$mpoint" $(echo $SYSDIRS $SDDIRS | tac -r -s '\s+')
    umountfs "$@"
}

remountfs() {
  local mpoint="$1"; shift
  ! is_bootname "$mpoint" || mpoint="$(get_root_mpoint "$mpoint")"
  mount -o remount "$@" "$mpoint"
}

fi #__included_mount_funcs
